# Emmy version 1.0.0
# EmmyFDSModel.py
# by Antoine Rauzy
# Copyrights (c) 2019 AltaRica Association
#
# This file is part of Emmy, a pedagogical stochastic simulator.
# Emmy is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Emmy is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Emmy.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creators
# 3) Access methods
# 4) Variables
# 5) Handles

# 1) Imported modules
# -------------------

from S2MLCore import *

from EmmyDecisionDiagrams import *

from EmmyFDSConstant import *
from EmmyFDSDomain import *
from EmmyFDSVariable import *
from EmmyFDSVariable import *

# 2) Creators
# -----------

class EmmyFDSModel:
  def __init__(self):
    from EmmyVariable import EmmyVariable
    self._fdsDomains = dict()
    self._fdsVariables = dict()
    self._fdsObservers = dict()

  def Reset(self):
    self._fdsDomains.clear()
    self._fdsVariables.clear()
    self._fdsObservers.clear()

# 3) Domains
# ----------

  def GetFDSDomains(self):
    return self._domains.values()

  def LookForFDSDomain(self, identifier):
    return self._fdsDomains.get(identifier, None)

  def NewFDSDomain(self, identifier, domain):
    fdsDomain = self.LookForFDSDomain(identifier)
    if fdsDomain==None:
      fdsDomain = EmmyFDSDomain(identifier, domain)
      self._fdsDomains[identifier] = fdsDomain
    return fdsDomain

# 4) Variables
# ------------

  def GetFDSVariables(self):
    return self._fdsVariables

  def LookForFDSVariable(self, identifier):
    return self._fdsVariables.get(identifier, None)

  def NewFDSVariable(self, identifier):
    fdsVariable = self.LookForFDSVariable(identifier)
    if fdsVariable==None:
      fdsVariable = EmmyFDSVariable(identifier)
      self._fdsVariables[identifier] = fdsVariable
    return fdsVariable

# 5) Observers
# ------------

  def GetFDSObservers(self):
    return self._fdsObservers

  def LookForFDSObserver(self, identifier):
    return self._fdsObservers.get(identifier, None)

  def NewFDSObserver(self, identifier):
    fdsObserver = self.LookForFDSObserver(identifier)
    if fdsObserver==None:
      fdsObserver = EmmyFDSVariable(identifier)
      self._fdsObservers[identifier] = fdsObserver
    return fdsObserver

