# S2MLCore version 1.0.0
# S2MLUniverse.py
# by Antoine Rauzy
# Copyrights (c) 2018 AltaRica Association
#
# This file is part of S2MLCore.
# S2MLCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# S2MLCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with S2MLCore.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Universes

# 1) Imported modules
# -------------------

from S2MLConstant import *

# 2) Universes
# ------------

class S2MLUniverse:
  def __init__(self):
    self._constants = dict()
    self._none = S2MLConstant.NewNone()
    self._false = S2MLConstant.NewBoolean(False)
    self._true = S2MLConstant.NewBoolean(True)

  def GetSize(self):
    return len(self._constants)

  def _GetConstant(self, type, value):
    return self._constants.get(value, None)

  def GetNone(self):
    return self._none
  def NewNone(self):
    return self._none
      
  def GetBoolean(self, value):
    if value:
      return self._true
    return self._false
  def NewBoolean(self, value):
    if value:
      return self._true
    return self._false

  def GetInteger(self, value):
    return self._GetConstant(S2MLConstant.INTEGER, value)
  def NewInteger(self, value):
    constant = self.GetInteger(value)
    if constant==None:
      constant = S2MLConstant.NewInteger(value)
      self._constants[value] = constant
    return constant

  def GetReal(self, value):
    return self._GetConstant(S2MLConstant.REAL, value)
  def NewReal(self, value):
    constant = self.GetReal(value)
    if constant==None:
      constant = S2MLConstant.NewReal(value)
      self._constants[value] = constant
    return constant

  def GetSymbol(self, value):
    return self._GetConstant(S2MLConstant.SYMBOL, value)
  def NewSymbol(self, value):
    constant = self.GetSymbol(value)
    if constant==None:
      constant = S2MLConstant.NewSymbol(value)
      self._constants[value] = constant
    return constant

  def GetString(self, value):
    return self._GetConstant(S2MLConstant.STRING, value)
  def NewString(self, value):
    constant = self.GetString(value)
    if constant==None:
      constant = S2MLConstant.NewString(value)
      self._constants[value] = constant
    return constant

  def GetConstant(self, constant):
    if constant.GetType()==S2MLConstant.NONE:
      return self.GetNone()
    if constant.GetType()==S2MLConstant.BOOLEAN:
      return self.GetBoolean(constant.GetValue())
    if constant.GetType()==S2MLConstant.INTEGER:
      return self.GetInteger(constant.GetValue())
    if constant.GetType()==S2MLConstant.REAL:
      return self.GetReal(constant.GetValue())
    if constant.GetType()==S2MLConstant.SYMBOL:
      return self.GetSymbol(constant.GetValue())
    if constant.GetType()==S2MLConstant.STRING:
      return self.GetString(constant.GetValue())
    return None
  def NewConstant(self, constant):
    if constant.GetType()==S2MLConstant.NONE:
      return self.NewNone()
    if constant.GetType()==S2MLConstant.BOOLEAN:
      return self.NewBoolean(constant.GetValue())
    if constant.GetType()==S2MLConstant.INTEGER:
      return self.NewInteger(constant.GetValue())
    if constant.GetType()==S2MLConstant.REAL:
      return self.NewReal(constant.GetValue())
    if constant.GetType()==S2MLConstant.SYMBOL:
      return self.NewSymbol(constant.GetValue())
    if constant.GetType()==S2MLConstant.STRING:
      return self.NewString(constant.GetValue())
    return None
