# S2MLCore version 1.0.0
# S2MLMatrix.py
# by Antoine Rauzy
# Copyrights (c) 2018 AltaRica Association
#
# This file is part of S2MLCore.
# S2MLCore is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# S2MLCore is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with S2MLCore.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Matrices

# 1) Imported modules
# -------------------

import string
import sys
import re
import math

from S2MLVector import *

# 2) Matrices
# -----------

class S2MLMatrix:
  def __init__(self):
    self._rows = []

  def GetNumberOfRows(self):
    return len(self._rows)
  def GetNumberOfColumns(self):
    numberOfColumns = 0
    for row in self._rows:
      size = row.GetSize()
      if size>numberOfColumns:
        numberOfColumns = size
    return numberOfColumns

  def AppendRow(self):
    row = S2MLVector()
    self._rows.append(row)
    return row
  def Extend(self, numberOfRows):
    while self.GetNumberOfRows()<numberOfRows:
      self.AppendRow()
  def Shrink(self, numberOfRows):
    while self.GetNumberOfRows()>numberOfRows:
      self._rows.pop()

  def GetRow(self, rowIndex, default=None):
    if rowIndex<0 or rowIndex>=self.GetNumberOfRows():
      return default
    return self._rows[rowIndex]
  def Get(self, rowIndex, columnIndex, default=None):
    if rowIndex<0 or rowIndex>=self.GetNumberOfRows():
      return default
    return self._rows[rowIndex].Get(columnIndex, default)
  def Set(self, rowIndex, columnIndex, constant):
    if rowIndex<0 or rowIndex>=self.GetNumberOfRows():
      return
    return self._rows[rowIndex].Set(columnIndex, constant)
  def Put(self, rowIndex, columnIndex, constant):
    if rowIndex<0:
      return
    if rowIndex>=self.GetNumberOfRows():
      self.Extend(rowIndex+1)
    return self._rows[rowIndex].Put(columnIndex, constant)
  def Reset(self, rowIndex, columnIndex):
    if rowIndex<0 or rowIndex>=self.GetNumberOfRows():
      return
    return self._rows[rowIndex].Reset(columnIndex)

